@@ -12,6 +12,7 @@ import os
 import getpass
 import mimetypes
 import requests
+import hashlib
 
 from pkg_resources import get_distribution
 from pkg_resources import DistributionNotFound
@@ -391,6 +392,30 @@ class SbSession:
         """
         return self.upload_files_and_update_item(item, [filename], scrape_file)
 
+    def upload_s3_files(self, itemid, s3_path, filenames):
+        """Upload a list of files from an external S3 bucket to an existing Item in ScienceBase
+
+        :param itemid: ScienceBase Catalog Item ID of the item to update
+        :param s3_path: External S3 bucket path, e.g. s3://mys3bucket/12 where files to be uploaded are located
+        :param filenames: List of filenames in external S3 bucket, located under s3_path, to be uploaded to Item
+        """
+        if not self._sbSessionEx.is_logged_in():
+            print(f'{self._username} not logged into Keycloak -- cloud services not available')
+        else:
+            try:
+                self.get_item(itemid)
+
+                input = {"filelist": filenames, "id": itemid, "key": s3_path}
+
+                response = self._sbSessionEx.upload_s3_files(input)
+
+                if response:
+                    print("Triggered upload of S3 files to ScienceBase Item")
+                else:
+                    print("Error uploading S3 files to ScienceBase Item")
+            except Exception:
+                print("Error triggering upload of S3 files to ScienceBase Item")
+
     def upload_cloud_file_to_item(self, itemid, filename):
         """Upload a file to cloud storage on an existing Item in ScienceBase
 
@@ -529,20 +554,34 @@ class SbSession:
         :return: The ScienceBase Catalog Item JSON of the updated Item
         """
         url = self._base_upload_file_url
+        checksums = []
         files = []
+        params = []
         for filename in filenames:
             if isinstance(filename, str):
                 if (os.access(filename, os.F_OK)):
                     files.append(('file', open(filename, 'rb')))
+                    checksums.append(self.get_file_checksum(filename))  
                 else:
                     raise Exception("File not found: " + filename)
             else:
                 files.append(('file', filename))
+
         data = {'item': json.dumps(item)}
         params = {} if scrape_file is True else {'scrapeFile':'false'}
         if 'id' in item and item['id']:
             data['id'] = item['id']
+            url = '{0}?id={1}&'.format(self._base_upload_file_url, item['id'])
+        else:
+            url = '{0}?'.format(self._base_upload_file_url)
+
+        for i, checksum in enumerate(checksums):
+            url += 'md5Checksum={0}'.format(checksum) if i == 0 else '&md5Checksum={0}'.format(checksum)
+
         ret = self._session.post(url, params=params, files=files, data=data)
+        # Close any open files
+        for f in files:
+            f[1].close()
         return self._get_json(ret)
 
     def upload_file(self, filename, mimetype=None):
@@ -558,14 +597,13 @@ class SbSession:
         url = self._base_upload_file_temp_url
 
         if os.access(filename, os.F_OK):
-            #
             # if no mimetype was sent in, try to guess
-            #
             if mimetype is None:
                 mimetype = self._guess_mimetype(filename)
             fname = os.path.basename(filename)
+            checksum = self.get_file_checksum(filename)
             with open(filename, 'rb') as f:
-                ret = self._session.post(url, files=[('files[]', (fname, f, mimetype))])
+                ret = self._session.post(url, params={'md5Checksum': checksum}, files=[('files[]', (fname, f, mimetype))])
                 retval = self._get_json(ret)
         else:
             raise Exception("File not found: " + filename)
@@ -624,8 +662,23 @@ class SbSession:
         itemfile['pathOnDisk'] = upld_json[0]['fileKey']
         itemfile['dateUploaded'] = upld_json[0]['dateUploaded']
         itemfile['uploadedBy'] = upld_json[0]['uploadedBy']
+        itemfile['checksum']= {'value': self.get_file_checksum(filename), 'type': 'MD5'}
+
         return itemfile
 
+    def get_file_checksum(self, filename):
+        """Get checksum value for a file (md5 Checksum from hashlib package)
+        
+        :param filename: File to get checksum on
+        """
+        with open(filename, 'rb') as f:
+            file_hash = hashlib.md5()
+            chunk = f.read(8192)
+            while chunk:
+                file_hash.update(chunk)
+                chunk = f.read(8192)
+            return file_hash.hexdigest()
+
     def delete_file(self, sb_filename, item):
         """Delete a file on a ScienceBase Item.  This method will delete all files with the provided
         name, whether they are in the files list or on a facet.
@@ -1275,8 +1328,6 @@ class SbSession:
             else:
                 print("Failed to publish file " + filename + " to public S3 bucket")
 
-
-
     def unpublish_array_from_public_bucket(self, item_id, filenames):
         """unpublish a list of files on an item from the public s3 publish bucket
 
@@ -1523,4 +1574,4 @@ class SbSession:
         :return: ItemLink JSON
         """
         related_item_link = self.get_item_link_type_by_name('related')
-        return self.create_item_link(from_item_id, to_item_id, related_item_link['id'])
+        return self.create_item_link(from_item_id, to_item_id, related_item_link['id'])
\ No newline at end of file
